//------------------------------------------------------------------------------
// File: AsyncFlt.h
//
// Desc: DirectShow sample code - header file for async filter.
//
// Copyright (c) 1996-2001 Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------

// Source filter for parsing RIFF/CDXA file and playing it on the fly (from a RIFF/CDXA cd)
// Original code from M$ DX8.1 SDK sample: samples\multimedia\dshow\filters\asynch
// Modified to RIFF/CDXA parsing by Avi Halachmi (avihpit@yahoo.com) 23-apr-2002
// dshow stuff, including automatic media type detection by De_xt


/* sample usage:
	  
    1. register the filter asyncflt.ax (shown in graphedit as 'RIFF/CDXA source') using register.bet
	2. create a RIFF/CDXA cd image using de_xt's tool that is containing at lease one media file. (the tool is attached)
	3. burn the image (or use deamon tools to mount it)
	4. run your favorite player and open a file from the new CD. it should play.
	5. if you want to remove this filter, use unregister.bat

    people had success with mp3, avi and ogg files.
	though it should support any media type that is playable with with windows media compatible player.
*/

/*


-- WARNING WARNING WARNING -- 
this is alpha software. this means that if you start storing files using de_xt's tool now, 
THE BURNED FILES WILL NOT BE COMPATIBLE WITH FUTURE VERSIONS OF THIS STORAGE FORMAT.
THE FORMAT IS NOT FINALIZED YET. (error correction hasn't been added yet) 

although it's possible to use these filters to read 800M files directly from the cd and play them,
you should NOT by any means start archiving your files using this format.
these filters are only here for testing of playback, co-existance with other filters,
smoothness of playback etc. 

the reason you should not start archiving your clips using de_xt's tool is that there's
no error correction. this means that if your cd is scratched, and 
important (=headers/indexes) parts of the file are lost, you will NOT be able to
play your file. 

de_xt's tool stores the files as standard m2f2 files on iso 9660 filesystem.
this mode offers no error correction. what is lost is lost. 

the final target is to add some error correction facilities, and then use the tool 
to make an image. the filter would also be modified to use these 
error correction facilities when errors are detected. 

atm though, THERE IS NO ERROR CORRECTION AND YOUR FILE MIGHT NEVER PLAY AGAIN IF THE
CD HAS SCRATCHES WHERE IT MATTERS.

be warned. 
*/

#ifndef avihdprintf
#define avihdprintf

#include <stdio.h>
#include <stdarg.h>

// usage is same as printf. output using OutputDebugString.
int dprintf(char* fmt, ...){
	char printString[1024];
	va_list argp;
	va_start(argp, fmt);
	vsprintf(printString, fmt, argp);
	va_end(argp);
    OutputDebugString(printString);
	return strlen(printString);
}
#endif


// {950AD390-D2DA-4029-BFD6-ECF019D3EF56}
DEFINE_GUID(CLSID_RIFFCDXAReader, 
0x950ad390, 0xd2da, 0x4029, 0xbf, 0xd6, 0xec, 0xf0, 0x19, 0xd3, 0xef, 0x56);

// {A92227E7-22F0-42f4-B6A4-C788839BB32F}
DEFINE_GUID(MEDIASUBTYPE_RIFF_CDXA,
0xa92227e7, 0x22f0, 0x42f4, 0xb6, 0xa4, 0xc7, 0x88, 0x83, 0x9b, 0xb3, 0x2f);

//
// foreign GUIDs -- TEST only!
//

// {D2855FA9-61A7-4db0-B979-71F297C17A04}
DEFINE_GUID(MEDIASUBTYPE_Ogg,0xd2855fa9, 0x61a7, 0x4db0, 0xb9, 0x79, 0x71, 0xf2, 0x97, 0xc1, 0x7a, 0x4);

// cddca2d5-6d75-4f98-840e-737bedd5c63b
DEFINE_GUID(MEDIASUBTYPE_Vorbis, 0xcddca2d5, 0x6d75, 0x4f98, 0x84, 0x0e, 0x73, 0x7b, 0xed, 0xd5, 0xc6, 0x3b);


#define SECTOR_SIZE			2352
#define RIFF_SIZE			44
#define SECTOR_HEADER		24
#define SECTOR_USER_DATA	2324
#define SECTOR_EDC			4


LONGLONG actualPosition(LONGLONG strippedPos);
LONGLONG strippedPosition(LONGLONG actualPosition);

//
//  Define an internal filter that wraps the base CBaseReader stuff
//

class CMemStream : public CAsyncStream
{
public:
    CMemStream();

    void Init(HANDLE hFile, LPBYTE pbData, LONGLONG llLength, DWORD dwKBPerSec);

    HRESULT SetPointer(LONGLONG llPos);
    HRESULT Read(PBYTE pbBuffer, DWORD dwBytesToRead, BOOL bAlign, LPDWORD pdwBytesRead);
    LONGLONG Size(LONGLONG *pSizeAvailable);
    DWORD Alignment();
    void Lock();
    void Unlock();

//    void strippedReadFile(void* dest, void* src, LONGLONG strippedPos, LONGLONG length);
    void strippedReadFile(void* dest, LONGLONG strippedPos, LONGLONG length);

    BOOL ReadFromFile(void *dest, LONGLONG pos, DWORD size);
private:

    CCritSec       m_csLock;
    PBYTE          m_pbData;
    LONGLONG       m_llLength;
    LONGLONG       m_llPosition;
    DWORD          m_dwKBPerSec;
    DWORD          m_dwTimeStart;
	LONGLONG	   m_strippedLength;
	LONGLONG	   m_strippedPosition;
	HANDLE		   m_hFile;
};


class CAsyncFilter : public CAsyncReader, public IFileSourceFilter
{
public:
    CAsyncFilter(LPUNKNOWN pUnk, HRESULT *phr);
    ~CAsyncFilter();

    static CUnknown *CreateInstance(LPUNKNOWN, HRESULT *);
    DECLARE_IUNKNOWN

    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv);
    STDMETHODIMP Load(LPCOLESTR lpwszFileName, const AM_MEDIA_TYPE *pmt);
    STDMETHODIMP GetCurFile(LPOLESTR * ppszFileName, AM_MEDIA_TYPE *pmt);

private:
//    BOOL ReadTheFile(LPCTSTR lpszFileName);
	BOOL OpenTheFile(LPCTSTR lpszFileName);
	BOOL DetectMediaType(BYTE *Data);
private:
    LPWSTR     m_pFileName;
    LONGLONG   m_llSize;
    PBYTE      m_pbData;
    CMemStream m_Stream;
	HANDLE	   m_hFile;
	PBYTE      m_pData;
	GUID       MediaTypeGUID;
	GUID       MediaSubtypeGUID;
};
